// this is a program for the Atmega32U4 microcontroller: generates
//  a PWM on the Timer4D output pin PD7, send a Hello\n message on
//  the RS232 port (whose TX port is connected to the RF switch control
//  pin to vary the load on the SAW filter from open to 50 ohms), and
//  communicates on the RS232 port. 

#define F_CPU 16000000UL
#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>  // _delay_ms
#include <avr/power.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdio.h>

#include "VirtualSerial.h"
#include <util/delay.h>

extern USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface;
extern FILE USBSerialStream;

#define USART_BAUDRATE 9600

// https://github.com/tomvdb/avr_arduino_leonardo/blob/master/examples/uart/main.c
void uart_transmit( unsigned char data )
{while (!(UCSR1A&(1<<UDRE1))) ;
 UDR1 = data;
}

unsigned char uart_receive(void)
{while (!(UCSR1A&(1<<RXC1))) ;
 return UDR1;
}

void pwm_configure()
{TCCR4A=0;
 TCCR4B=(1<<CS40); // 48000000/512=10.66 us
 TCCR4C=0;
 TCCR4D=0;
 PLLFRQ=(PLLFRQ&0xCF)|0x30; // PLL_FREQ 48000000UL
 TC4H=0x00;                 // compteur sur 10 bits (hi)
 OCR4C=250;                 // Terminal count for Timer 4 PWM: val max
}

void pwmD_configure() // Timer4 D
{OCR4D=0;      // Set PWM value: init @ 0
 DDRD|=1<<7;   // Set Output Mode BIEN QUE PWM, DOIT ETRE OUTPUT
 TCCR4C|=0x09; // Activate channel D
}

/*
void pwmA_configure() // Timer4 A
{
 OCR4A=0;      // Set PWM value: init @ 0
 DDRC|=1<<7;   // Set Output Mode 
 TCCR4A=0x82;  // Activate channel A
}
*/

int main(void)
{ unsigned short baud;
  wdt_disable();
  DDRD=0x00;

  UCSR1A = 0;                         // importantly U2X1 = 0
  UCSR1B = 0;                         
  UCSR1B = (1 << RXEN1)|(1 << TXEN1); // enable receiver and transmitter
  UCSR1C = _BV(UCSZ11) | _BV(UCSZ10); // 8N1
  // UCSR1D = 0;                         // no rtc/cts (probleme de version de libavr)
  baud  = (((( F_CPU / ( USART_BAUDRATE * 16UL))) - 1));
  UBRR1H = (unsigned char)(baud>>8);
  UBRR1L = (unsigned char)baud;

 SetupHardware();
/* Create a regular character stream for the interface so that it can be used with the stdio.h functions */
  CDC_Device_CreateStream(&VirtualSerial_CDC_Interface, &USBSerialStream);
  GlobalInterruptEnable();

  pwm_configure();
  pwmD_configure();     // 4D
  TC4H=(1)>>8;
  OCR4D=(char)(25*2);    // OCR4A pour le A
  while(1) {
//     fprintf(&USBSerialStream,"Hello\r\n");
     uart_transmit('H');
     uart_transmit('e');
     uart_transmit('l');
     uart_transmit('l');
     uart_transmit('o');
     uart_transmit('\n');
     _delay_ms(150);
     CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);
     CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
     USB_USBTask();
    }
}
