/* -*- c++ -*- */
/* 
 * Copyright 2016 <+YOU OR YOUR COMPANY+>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "serial_impl.h"

namespace gr {
  namespace theremin {

    serial::sptr
    serial::make(int step)
    {
      return gnuradio::get_initial_sptr
        (new serial_impl(step));
    }

    /*
     * The private constructor
     */
    serial_impl::serial_impl(int step)
      : sync_block("serial",
              gr::io_signature::make(1, 1, sizeof(float)),
              gr::io_signature::make(0, 0, 0)),_next_pos(0), 
			  _curr_char('\0'), _step(step),
			  _nb_bits(0), _in_frame(0),
			  test_pos(false), _total(0)
    {
		printf("hello world\n");		
	}

    /*
     * Our virtual destructor.
     */
    serial_impl::~serial_impl()
    {
    }

    void
    serial_impl::forecast (int noutput_items, gr_vector_int &ninput_items_required)
    {
        /* <+forecast+> e.g. ninput_items_required[0] = noutput_items */
    }
#define HIGH_STATE 1
#define LOW_STATE 0
    int
    serial_impl::work(int noutput_items,
                       gr_vector_const_void_star &input_items,
                       gr_vector_void_star &output_items)
    {
        const float *in = (const float *) input_items[0];
		volatile long i = _next_pos;
		int start_step = _step + (_step>>1);
		unsigned char bit;

		//printf("noutput_items : %d\n", noutput_items);
		while (i < noutput_items) {
			bit = (unsigned char)in[i] ^ 1;
			if (_in_frame == 0) { 
			/* not in a frame :
			 * search for start bit falling edge
			 */
				if (bit == LOW_STATE) {
					_in_frame = 1;
					/* 1/2 period for start bit + 1 period to be 
					 * in the middle of first bit
					 */
					i += start_step;
					_total += start_step;
				} else {
					i++; 
					_total ++;
				}
			} else {
				if (_nb_bits >= 8) { /* stop bit == hi ? */
                                   if (bit==HIGH_STATE)
                                        {if ((_curr_char>31)&&(_curr_char))
						printf("%c", (unsigned char)_curr_char);
					 else
						printf("(%x)\n", (unsigned char)_curr_char);
                                        }
				    _curr_char = '\0';
				    _nb_bits = 0;
				    _next_pos = 0;
				    _in_frame = 0;
				} else {
					/* add bit to current char */
					_curr_char |= (bit << _nb_bits);
					_nb_bits++;
					i += _step;
					_total += _step;
				}
			}
		}
		/* if trame is on multiple buffer */
		if (_in_frame == 1)
			_next_pos = i - noutput_items;
        consume_each (noutput_items);
		//return 0;
        return noutput_items;// pos;
    }

  } /* namespace theremin */
} /* namespace gr */

