/* -*- c++ -*- */
/* 
 * Copyright 2016 <+YOU OR YOUR COMPANY+>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "th_mean_impl.h"

namespace gr {
  namespace theremin {

    th_mean::sptr
    th_mean::make(int mean_length, float threshold)
    {
      return gnuradio::get_initial_sptr
        (new th_mean_impl(mean_length, threshold));
    }

    /*
     * The private constructor
     */
    th_mean_impl::th_mean_impl(int mean_length, float threshold)
      : gr::block("th_mean",
      //: sync_decimator("th_mean",
      //: sync_block("th_mean",
              gr::io_signature::make(1, 1, sizeof(float)),
              gr::io_signature::make(0, 1, sizeof(float))/*,2*/),
			  _threshold(threshold),
			  _mean_length((float)mean_length), _current_pos(0),
			  _list_mean(mean_length), _mean(0.0f)
    {}

    /*
     * Our virtual destructor.
     */
    th_mean_impl::~th_mean_impl()
    {
    }

    void
    th_mean_impl::forecast (int noutput_items, gr_vector_int &ninput_items_required)
    {
        /* <+forecast+> e.g. ninput_items_required[0] = noutput_items */
    }

    int
    th_mean_impl::general_work (int noutput_items,
    //th_mean_impl::work (int noutput_items,
                       gr_vector_int &ninput_items,
                       gr_vector_const_void_star &input_items,
                       gr_vector_void_star &output_items)
    {
        const float *in = (const float *) input_items[0];
        float *out = (float *) output_items[0];
		float current;
		int pos = 0;

		float back_value = 0;
		for (int i=0; i<noutput_items; i++) {
			current = (in[i] <= _threshold) ? 0.0f : in[i];
			if (current != 0.0f) {
				back_value = _list_mean.front();
				_list_mean.pop_front();
				_list_mean.push_back(current);
				_mean+= current - back_value;
			}
			out[pos] = ((_mean/_mean_length));
			pos++;

		}
        consume_each (noutput_items);
		return pos;

        // Tell runtime system how many output items we produced.
        //return noutput_items;
    }

	void th_mean_impl::set_threshold(float threshold)
	{
		_threshold = threshold;
	}

	float th_mean_impl::threshold()
	{
		return _threshold;
	}

  } /* namespace theremin */
} /* namespace gr */

