/* -*- c++ -*- */
/* 
 * Copyright 2016 <+YOU OR YOUR COMPANY+>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "theremin_impl.h"

namespace gr {
	namespace theremin {

		theremin::sptr theremin::make(float threshold) {
			return gnuradio::get_initial_sptr
			    (new theremin_impl(threshold));
		}
		/*
		 * The private constructor
		 */ theremin_impl::theremin_impl(float threshold)
		:gr::block("theremin",
			   gr::io_signature::make(1, 1, sizeof(float)),
			   gr::io_signature::make(1, 1, sizeof(float))),
		    _threshold(threshold), _nb_accum(0), _accum(0),
		    _old_current(0), _init_state(true)
		{
		}
		/*
		 * Our virtual destructor.
		 */ theremin_impl::~theremin_impl() {
		}

		void
		 theremin_impl::forecast(int noutput_items,
					 gr_vector_int & ninput_items_required)
		{
			/* <+forecast+> e.g. ninput_items_required[0] = noutput_items */
		}

		int
		 theremin_impl::general_work(int noutput_items,
					     gr_vector_int & ninput_items,
					     gr_vector_const_void_star &
					     input_items,
					     gr_vector_void_star & output_items)
		{
			int pos = 0;
			float current;
			const float *in = (const float *)input_items[0];
			float *out = (float *)output_items[0];

			for (int i = 0; i < noutput_items; i++) {
				current = (in[i] < _threshold) ? 0 : in[i];

				if (_init_state) {	// avoid accum if start is in transmission
					if (current == 0)
						_init_state = false;
				} else {
					if (current != 0) {
						_accum += current;
						_nb_accum++;
					} else {
						/* transmit mean only on falling edge */
						if (_old_current != 0) {
							out[pos] =
							    _accum / _nb_accum;
							_accum = 0;
							_nb_accum = 0;
							pos++;
						}
					}
				}

				_old_current = current;
			}

			// Do <+signal processing+>
			// Tell runtime system how many input items we consumed on
			// each input stream.
			consume_each (noutput_items);
			return pos;

			// Tell runtime system how many output items we produced.
			return noutput_items;
		}

	}			/* namespace theremin */
}				/* namespace gr */
